package BalanceOfPower::Relations::Alliance;
$BalanceOfPower::Relations::Alliance::VERSION = '0.300040';
#DEPRECATED

use strict;
use v5.10;

use Moo;

with 'BalanceOfPower::Relations::Role::Relation';


1;
